#uses "fwUnitTestComponentAsserts.ctl"

#uses "fwRPC/fwRPC_RingBuffer.ctl"

private global string dp_rb = "rbtest";

void test_fwRPC_RingBuffer_setup() {
  if(dpExists(dp_rb)) dpDelete(dp_rb);
}

void test_fwRPC_RingBuffer_init() {
  fwRPC_RingBuffer_init(dp_rb, 4);

  assertTrue(dpExists(dp_rb));

  uint read,write,size;
  dpGet(
    dp_rb + fwRPC_RingBuffer_DPE_READ,  read,
    dp_rb + fwRPC_RingBuffer_DPE_WRITE, write,
    dp_rb + fwRPC_RingBuffer_DPE_SIZE,  size
  );

  assertEqual(1, read);
  assertEqual(1, write);
  assertEqual(4, size);

  dpSetWait(
    dp_rb + fwRPC_RingBuffer_DPE_READ,  3,
    dp_rb + fwRPC_RingBuffer_DPE_WRITE, 4,
    dp_rb + fwRPC_RingBuffer_DPE_SIZE,  6
  );

  fwRPC_RingBuffer_init(dp_rb, 5);

  dpGet(
    dp_rb + fwRPC_RingBuffer_DPE_READ,  read,
    dp_rb + fwRPC_RingBuffer_DPE_WRITE, write,
    dp_rb + fwRPC_RingBuffer_DPE_SIZE,  size
  );

  assertEqual(1, read);
  assertEqual(1, write);
  assertEqual(5, size);
}

void test_fwRPC_RingBuffer_writeRead() {
  string val;

  assertEqual(-1, fwRPC_RingBuffer_write(dp_rb,"x"), "RB wasn't initialized.");
  assertEqual(-1, fwRPC_RingBuffer_read(dp_rb,val), "RB wasn't initialized.");

  fwRPC_RingBuffer_init(dp_rb, 4);

  assertEqual(0, fwRPC_RingBuffer_write(dp_rb,"v1"), "Value should be written.");
  assertEqual(0, fwRPC_RingBuffer_write(dp_rb,"v2"), "Value should be written.");
  assertEqual(0, fwRPC_RingBuffer_write(dp_rb,"v3"), "Value should be written.");
  assertEqual(0, fwRPC_RingBuffer_write(dp_rb,"v4"), "Value should be written.");
  assertEqual(-2, fwRPC_RingBuffer_write(dp_rb,"v5"), "Value not should be written.");
  assertEqual(0, fwRPC_RingBuffer_read(dp_rb, val), "Value should be read.");
  assertEqual("v1", val);
  assertEqual(0, fwRPC_RingBuffer_write(dp_rb,"v6"), "Value should be written.");
  assertEqual(-2, fwRPC_RingBuffer_write(dp_rb,"v7"), "Value should not be written.");
  assertEqual(0, fwRPC_RingBuffer_read(dp_rb, val), "Value should be read.");
  assertEqual("v2", val);
  assertEqual(0, fwRPC_RingBuffer_read(dp_rb, val), "Value should be read.");
  assertEqual("v3", val);
  assertEqual(0, fwRPC_RingBuffer_read(dp_rb, val), "Value should be read.");
  assertEqual("v4", val);
  assertEqual(0, fwRPC_RingBuffer_read(dp_rb, val), "Value should be read.");
  assertEqual("v6", val);
  assertEqual(-2, fwRPC_RingBuffer_read(dp_rb, val), "There should be no more values.");
}


void test_fwRPC_RingBuffer_write_many() {
  dyn_string write_values = makeDynString("str a", "str b", "str c", "str d");
  dyn_string read_values = makeDynString();
  int err;

  err = fwRPC_RingBuffer_init(dp_rb, 4);
  assertEqual(fwRPC_RingBuffer_ESUCCESS, err);

  int inserted;

  //  try writing none
  err = fwRPC_RingBuffer_writeMany(dp_rb, write_values, 0, inserted);
  assertEqual(fwRPC_RingBuffer_ESUCCESS, err);
  assertEqual(0, inserted);

  err = fwRPC_RingBuffer_readMany(dp_rb, read_values, 1);
  assertEqual(fwRPC_RingBuffer_EEMPTY, err);
  assertEqual(0, dynlen(read_values));

  //  try writing 1
  err = fwRPC_RingBuffer_writeMany(dp_rb, write_values, 1, inserted);
  assertEqual(fwRPC_RingBuffer_ESUCCESS, err);
  assertEqual(1, inserted);
  
  err = fwRPC_RingBuffer_readMany(dp_rb, read_values, 1);
  assertEqual(fwRPC_RingBuffer_ESUCCESS, err);
  assertEqual(1, dynlen(read_values));

  err = fwRPC_RingBuffer_readMany(dp_rb, read_values, 1);
  assertEqual(fwRPC_RingBuffer_EEMPTY, err);

  
  //  try writing 2
  err = fwRPC_RingBuffer_writeMany(dp_rb, write_values, 2, inserted);
  assertEqual(fwRPC_RingBuffer_ESUCCESS, err);
  assertEqual(2, inserted);

  err = fwRPC_RingBuffer_readMany(dp_rb, read_values, 4);
  assertEqual(fwRPC_RingBuffer_ESUCCESS, err);
  assertEqual(2, dynlen(read_values));

  err = fwRPC_RingBuffer_readMany(dp_rb, read_values, 1);
  assertEqual(fwRPC_RingBuffer_EEMPTY, err);

  //  try writing all with -1
  err = fwRPC_RingBuffer_writeMany(dp_rb, write_values, -1, inserted);
  assertEqual(fwRPC_RingBuffer_ESUCCESS, err);
  assertEqual(4, inserted, "(write all) should insert 4");

  err = fwRPC_RingBuffer_readMany(dp_rb, read_values, 4);
  assertEqual(fwRPC_RingBuffer_ESUCCESS, err);
  assertEqual(4, dynlen(read_values), "(write all) should read 4");

  err = fwRPC_RingBuffer_readMany(dp_rb, read_values, 1);
  assertEqual(fwRPC_RingBuffer_EEMPTY, err);

  //  try writing over
  err = fwRPC_RingBuffer_writeMany(dp_rb, write_values, 7, inserted);
  assertEqual(fwRPC_RingBuffer_ESUCCESS, err);
  assertEqual(4, inserted, "(write over) should insert 4");

  err = fwRPC_RingBuffer_readMany(dp_rb, read_values, 7);
  assertEqual(fwRPC_RingBuffer_ESUCCESS, err);
  assertEqual(4, dynlen(read_values), "(write over) should read 4");

  err = fwRPC_RingBuffer_readMany(dp_rb, read_values, 1);
  assertEqual(fwRPC_RingBuffer_EEMPTY, err);


  //  try with smaller rb bounds
  err = fwRPC_RingBuffer_init(dp_rb, 3);
  assertEqual(fwRPC_RingBuffer_ESUCCESS, err);

  err = fwRPC_RingBuffer_writeMany(dp_rb, write_values, -1, inserted);
  assertEqual(fwRPC_RingBuffer_ESUCCESS, err);
  assertEqual(3, inserted, "(write all bounds) should insert 3");

  err = fwRPC_RingBuffer_readMany(dp_rb, read_values, 4);
  assertEqual(fwRPC_RingBuffer_ESUCCESS, err);
  assertEqual(3, dynlen(read_values), "(write all bounds) should read 3");

  err = fwRPC_RingBuffer_readMany(dp_rb, read_values, 1);
  assertEqual(fwRPC_RingBuffer_EEMPTY, err);
}


void test_fwRPC_RingBuffer_read_many() {
  dyn_string write_values = makeDynString("str a", "str b", "str c");
  dyn_string read_values = makeDynString();
  int err;

  err = fwRPC_RingBuffer_init(dp_rb, 3);
  assertEqual(fwRPC_RingBuffer_ESUCCESS, err);

  err = fwRPC_RingBuffer_write(dp_rb, write_values[1]);
  assertEqual(fwRPC_RingBuffer_ESUCCESS, err);
  err = fwRPC_RingBuffer_write(dp_rb, write_values[2]);
  assertEqual(fwRPC_RingBuffer_ESUCCESS, err);
  err = fwRPC_RingBuffer_write(dp_rb, write_values[3]);
  assertEqual(fwRPC_RingBuffer_ESUCCESS, err);

  err = fwRPC_RingBuffer_readMany(dp_rb, read_values, 2);
  assertEqual(fwRPC_RingBuffer_ESUCCESS, err);

  assertEqual(2, dynlen(read_values));
  assertEqual(write_values[1], read_values[1]);
  assertEqual(write_values[2], read_values[2]);

  err = fwRPC_RingBuffer_readMany(dp_rb, read_values, 2);
  assertEqual(fwRPC_RingBuffer_ESUCCESS, err);

  assertEqual(1, dynlen(read_values));
  assertEqual(write_values[3], read_values[1]);

  err = fwRPC_RingBuffer_readMany(dp_rb, read_values, 1);
  assertEqual(fwRPC_RingBuffer_EEMPTY, err);
}
