#uses "fwUnitTestComponentAsserts.ctl"
#uses "test/fwRPC/utils.ctl"

#uses "fwRPC/fwRPC_workerManager.ctl"


void test_fwRPC_workerManager_setupSuite() {
  _BEGIN_SUITE("worker manager");
}

void test_fwRPC_workerManager_teardownSuite() {
  _END_SUITE();
}

void test_fwRPC_workerManager_setup() {
  _BEGIN_TEST();

  mappingClear(_workerEventList);
}

void test_fwRPC_workerManager_teardown() {

}


//------------------------------------------------------------------------------


private global const string _CALLBACK = "_workerDeathCB";
private global mapping _workerEventList;

void _workerDeathCB(int workerId) {
  _workerEventList[workerId] = true;
}


//------------------------------------------------------------------------------


void test_fwRPC_workerManager_detects_worker_death_and_evokes_cb() {
  fwRPC_workerManager_initialize(-1, -1, _CALLBACK);

  dyn_mixed worker = fwRPC_workerModel_new(1);
  int workerId = worker[fwRPC_WORKER_MODEL_ID];
  fwRPC_workerManager_registerWorker(worker);

  delay(fwRPC_PMON_STATE_WAIT_TIMEOUT + (fwRPC_WORKER_HEARTBEAT_RATE_MS / 1000) % 60 + 2);

  assertTrue(mappingHasKey(_workerEventList, workerId), "worker event should exist");
  assertTrue(_workerEventList[workerId]);

  mappingClear(_workerEventList);
  fwRPC_workerManager_deleteWorker(worker[workerId]);

  fwRPC_workerManager_registerWorker(worker);

  //Check ok for 3 ticks
  for(int i=0; i<3; ++i) {
    fwRPC_workerManager_notifyHeartbeat(worker[fwRPC_WORKER_MODEL_ID]);
    assertFalse(mappingHasKey(_workerEventList, workerId), "should not have event for worker");
    delay((fwRPC_WORKER_HEARTBEAT_RATE_MS/1000)%60, fwRPC_WORKER_HEARTBEAT_RATE_MS%1000);
  }

  assertFalse(mappingHasKey(_workerEventList, workerId), "should not have event for worker (2)");

  delay(fwRPC_WORKER_HEARTBEAT_WAIT_S, fwRPC_WORKER_HEARTBEAT_RATE_MS * 2);

  assertTrue(mappingHasKey(_workerEventList, workerId), "should have death event for worker");
}


void test_fwRPC_workerManager_adds_and_deletes_new_workers() {
  fwRPC_workerManager_initialize(-1, -1, _CALLBACK);

  dyn_mixed worker = fwRPC_workerModel_new(1);
  assertEqual(0, fwRPC_workerManager_registerWorker(worker));
  assertEqual(-1, fwRPC_workerManager_registerWorker(worker));
  assertEqual(0, fwRPC_workerManager_deleteWorker(worker[fwRPC_WORKER_MODEL_ID]));
  assertEqual(-1, fwRPC_workerManager_deleteWorker(worker[fwRPC_WORKER_MODEL_ID]));
}


void test_fwRPC_workerManager_finds_suitable_worker() {
  fwRPC_workerManager_initialize(-1, -1, _CALLBACK);

  dyn_mixed newWorker;
  assertEqual(-1, fwRPC_workerManager_findSuitableWorker(false, newWorker));
  assertEqual(0, dynlen(newWorker));

  dyn_mixed worker = fwRPC_workerModel_new(1);
  assertEqual(0, fwRPC_workerManager_registerWorker(worker));

  assertEqual(0, fwRPC_workerManager_findSuitableWorker(false, newWorker));
  assertTrue(fwRPC_workerModel_isValid(newWorker), "worker should be valid");
  assertEqual(1, newWorker[fwRPC_WORKER_MODEL_ID]);
}


void test_fwRPC_workerManager_sets_workers_as_pending_restart() {
  fwRPC_workerManager_initialize(-1, -1, _CALLBACK);

  dyn_mixed worker1 = fwRPC_workerModel_new(1);
  dyn_mixed worker2 = fwRPC_workerModel_new(2);

  assertEqual(0, fwRPC_workerManager_registerWorker(worker1));
  assertEqual(0, fwRPC_workerManager_registerWorker(worker2));
  assertEqual(2, fwRPC_workerManager_getWorkerCount());

  assertFalse(fwRPC_workerManager_isAnyWorkerPendingRestart());

  assertFalse(fwRPC_workerManager_isWorkerPendingRestart(worker1));
  assertFalse(fwRPC_workerManager_isWorkerPendingRestart(worker2));

  fwRPC_workerManager_pendWorkerRestart(false);

  assertTrue(fwRPC_workerManager_isAnyWorkerPendingRestart());
  assertTrue(fwRPC_workerManager_isWorkerPendingRestart(worker1));
  assertTrue(fwRPC_workerManager_isWorkerPendingRestart(worker2));

  //TODO: stop pending restart
  //      check counts
}
